# -*- coding: utf-8 -*-

import os
import html
import mimetypes
from wsgi import http, tools
from wsgi.middlewares.highlighter import HighlighterMiddleware
from wsgi.middlewares.chrome import ChromeMiddleware

class Application:

    def __init__(self, environ, start_response):
        self.environ = environ
        self.start_response = start_response
        
    def __iter__(self):
        path_info = self.environ.get('PATH_INFO', '/')
        if path_info == '/': path_info = '/index/'
        
        if path_info[-1] != '/':
            http.path_redirect(self.environ, self.start_response, path_info[1:] + '/')
            return
               
        parts = path_info.split('/')[1:]
        page = []
        for part in parts:
            if part and part != '..' and part != '.':
                page.append(part)
        part = page[0]
        parts = page
        page = '/'.join(page)
        
        # check module
        filename = os.path.dirname(__file__) + '/modules/' + part + '.py'
        if os.path.exists(filename):
            app = __import__('modules.%s' % part, locals(), globals(), ['run']).run
            for line in app(self.environ, self.start_response, parts):
                yield line
            return
        
        # otherwise check static page
        filename = os.path.dirname(__file__) + '/pages/' + page + '.txt'
        if not os.path.exists(filename):
            filename = os.path.dirname(__file__) + '/pages/error.txt'
        
        fp = file(filename)
        text = ''
        
        for line in fp:
            text += line
        
        self.start_response('200 OK', [('Content-Type', 'text/html')])
        title, text = html.parse(text)
        yield html.header(title, part)
        yield text
        yield html.footer()

chrome = {
    '/chrome':      os.path.dirname(__file__) + '/static',
    '/images':      os.path.dirname(__file__) + '/attachments/images',
    '/videos':      os.path.dirname(__file__) + '/attachments/videos',
    '/files':       os.path.dirname(__file__) + '/attachments/files',
    '/raw':         os.path.dirname(__file__) + '/pages'
}

app = Application
app = ChromeMiddleware(app, chrome)
app = HighlighterMiddleware(app)
