# -*- coding: utf-8 -*-

import re
from md5 import md5
from random import random
from xml.sax import saxutils

def header(title, part=''):
    menu_items = [
        ('index',       'Übersicht'),
        ('news',        'News'),
        ('verein',      'Verein'),
        ('events',      'Events'),
        ('beitreten',   'Beitreten'),
        ('links',       'Links'),
        ('suche',       'Suche'),
        ('impressum',   'Impressum')
    ]
    menubar = []
    for href, menutitle in menu_items:
        menubar.append('<li%s><a href="/%s/">%s</a></li>' % (
            ((href == part) and ' class="active"' or ''),
            href, menutitle))
    menubar = '\n'.join(menubar)
    return '''
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title>ubuntu Deutschland e.V. / %(title)s</title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<link rel="stylesheet" href="/chrome/screen.css" media="screen" />
<link rel="stylesheet" href="/chrome/print.css" media="print" />
<script type="text/javascript" src="/chrome/admin.js"></script>
</head>
<body onload="init()">

<div id="header">
    <img src="/chrome/banner.png" alt="Logo" />
    <span>ubuntu Deutschland e.V.</span>
    <ul id="navigation">
        %(menubar)s
    </ul>
</div>

''' % {'title': title, 'menubar': menubar}

def footer():
    return '''
<div id="footer">powered by <a href="http://wsgiarea.pocoo.org/">WSGIarea</a></div>

</body>
</html>'''

def format_mail(mail, caption=None):
    def format(string):
        string = string.replace('.', ' <strong>dot</strong> ')
        string = string.replace('-', ' <strong>dash</strong> ')
        string = string.replace('_', ' <strong>underline</strong> ')
        return string
    
    username, domain = mail.split('@')
    if caption is None:
        caption = format(username) + ' <strong>at</strong> ' + format(domain)
        replace_domain = ' el.innerHTML = "%s" + "@" + "%s";' % (username, domain)
    else:
        replace_domain = ''
    linkid = 'mail-' + md5(str(random())).hexdigest()
    
    return ''.join([
        '<a href="#" id="%s">%s</a>' % (linkid, caption),
        '<script type="text/javascript">',
        ' el = document.getElementById("%s");' % linkid,
        ' el.href = "mail" + "to:" + "%s" + "@" + "%s";' % (username, domain),
        replace_domain,
        '</script>'
    ])

def parse(content):
    blocks = []
    toc = []
    headid = []

    def do_code(matchobj):
        id = md5(str(random())).hexdigest()
        blocks.append(('code', id, matchobj.group(1)))
        return id
        
    def do_ul(matchobj):
        result = []
        tmp = matchobj.group(0)
        while True:
            matchobj2 = re.search('^  \*(.*?)$(?uism)', tmp)
            if matchobj2 is None:
                break
            result.append('<li>%s</li>' % matchobj2.group(1))
            tmp = tmp.replace(matchobj2.group(0), '', 1)
        return '</p><ul>' + ('\n'.join(result)) + '</ul><p>'
        
    def do_ol(matchobj):
        result = []
        tmp = matchobj.group(0)
        while True:
            matchobj2 = re.search('^  \-(.*?)$(?uism)', tmp)
            if matchobj2 is None:
                break
            result.append('<li>%s</li>' % matchobj2.group(1))
            tmp = tmp.replace(matchobj2.group(0), '', 1)
        return '</p><ol>' + ('\n'.join(result)) + '</ol><p>'
        
    def do_link(matchobj):
        parts = matchobj.group(1).split('|', 1)
        if len(parts) > 1:
            href = parts[0].strip()
            title = parts[1].strip()
        else:
            href = parts[0].strip()
            title = parts[0].strip()
        id = md5(str(random())).hexdigest()
        blocks.append(('link', id, href, title))
        return id
        
    def do_image(matchobj):
        lchar = matchobj.group(1)[0]
        rchar = matchobj.group(1)[-1]
        
        if lchar == ' ' and rchar == ' ':
            iclass = 'center'
        elif lchar == ' ':
            iclass = 'left'
        elif rchar == ' ':
            iclass = 'right'
        else:
            iclass = 'default'
        
        parts = matchobj.group(1).split('|', 1)
        if len(parts) > 1:
            image = '<a href="%s"><img class="image_%s" src="%s" alt="%s" /></a>' % (
                    parts[1].strip(), iclass, parts[0].strip(), parts[0].strip())
        else:
            image = '<img class="image_%s" src="%s" alt="%s" />' % (
                    iclass, parts[0].strip(), parts[0].strip())
        id = md5(str(random())).hexdigest()
        blocks.append(('raw', id, image))
        return id
        
    def do_mail(matchobj):
        parts = matchobj.group(1).split('|', 1)
        if len(parts) > 1:
            data = format_mail(parts[0].strip(), parts[1].strip())
        else:
            data = format_mail(parts[0].strip())
        id = md5(str(random())).hexdigest()
        blocks.append(('raw', id, data))
        return id
        
    def do_head(matchobj):
        if len(matchobj.group(1)) == len(matchobj.group(3)):
            id = md5(str(random())).hexdigest()
            level = len(matchobj.group(1))
            headid.append(True)
            nid = str(len(headid))
            blocks.append(('head', id, level, nid, matchobj.group(2).strip()))
            toc.append(('&nbsp; ' * (level - 1)) + '- <a href="#head-' + nid + '">' + matchobj.group(2) + '</a>')
            return id
        return matchobj.group(0)

    # kill windows eol chars and remove trailing whitespaces
    # and skip comments
    lines = content.splitlines()
    content = ''
    for line in lines:
        if not line.startswith('##'):
            content += line.rstrip() + '\n'
    content = '\n\n%s\n\n' % content

    # replace newlines
    content = re.sub(r'\\\\\s(?uism)', '<br />\n', content)
    
    rules = [
        (r'^(=+)(.*?)(=+)$(?uism)', do_head),
        (r'\{\{\{(.*?)\}\}\}(?uism)', do_code),
        (r'>>>(.*?)<<<(?uism)', r'</p><blockquote><p>\1</p></blockquote><p>'),
        (r'\[\[\s*mailto:(.*?)\s*\]\](?uism)', do_mail),
        (r'\[\[\s*(.*?)\s*\]\](?uism)', do_link),
        (r'\{\{(.*?)\}\}(?uism)', do_image),
        (r'^  \*(.*?)$\n\n(?uism)', do_ul),
        (r'^  \-(.*?)$\n\n(?uism)', do_ol),
        (r'\*\*(.*?)\*\*(?uism)', r'<strong>\1</strong>'),
        (r'//(.*?)//(?uism)', r'<em>\1</em>'),
        (r'__(.*?)__(?uism)', r'<ins>\1</ins>'),
        (r'---+', r'</p><hr /><p>'),
        (r'\r?\n\r?\n(?uism)', '</p>\n<p>\n'),
    ]
    for s, r in rules:
        content = re.sub(s, r, content)
        
    # restore blocks
    for area in blocks:
        if area[0] == 'raw':
            content = content.replace(area[1], area[2])
        if area[0] == 'code':
            content = content.replace(area[1], '</p><pre>%s</pre><p>' % saxutils.escape(area[2]))
        if area[0] == 'head':
            content = content.replace(area[1], '</p>\n<h%i id="head-%s">%s</h%i>\n<p>' % (
                                      area[2], area[3], area[4], area[2]))
        if area[0] == 'link':
            content = content.replace(area[1], '<a href="%s">%s</a>' % (area[2], area[3]))

    # strip empty paragraphs
    content = re.sub(r'<p>\s*</p>(?uism)', '', '<p>%s</p>' % content)
    
    # toc
    if len(toc) > 2:
        toc = '<div id="toc"><h2>Inhalt</h2><p>%s</p></div>' % '<br />'.join(toc)
        content = toc + content
    
    # title
    title = False
    for block in blocks:
        if not title and block[0] == 'head':
            title = block[4]
    if not title:
        title = 'untitled'
    
    return (title, content)
