# -*- coding: utf-8 -*-

from sqlobject import *
from datetime import datetime
from xml.sax import saxutils
from random import random
from md5 import md5

SQLObject._connection = connectionForURI('sqlite:///tmp/ubuntu-de-verein.db')


class Mitglied(SQLObject):

    uid = StringCol(length=30, alternateID=True)
    name = StringCol(length=300)
    password = StringCol(length=32)
    GPGkeyid = StringCol(length=40)
    username = StringCol(length=200)
    ircname = StringCol(length=60)
    mail = StringCol(length=300)
    telefon = StringCol(length=30)
    mitgliedsart = IntCol()
    bezahlt_bis = DateTimeCol()
    adresse = StringCol(length=2048)
    kontonummer = StringCol(length=256)
    mahnstufe = IntCol()
    private_infos = IntCol()
    is_admin = BoolCol()
    
    def getDisplayName(self):
        if self.private_infos == 0:
            return 'anonymous'
        if self.private_infos == 1:
            return self.username.split('@')[0]
        return ' '.join(reversed(self.name.split(' ', 1)))
        
    def getForumLink(self):
        if not self.username:
            return ''
        username, forum = self.username.split('@')
        link = '#'
        if forum == 'ubuntuusers.de':
            link = 'http://www.ubuntuusers.de/user/' + saxutils.escape(username)
        return '<a href="%s">%s</a>' % (link, username)
        
    def formatGPGKey(self):
        key = ''
        i = 0
        for char in self.GPGkeyid:
            key += char
            i += 1
            if i == 4:
                key += ' '
                i = 0
        short = self.GPGkeyid[-16:]
        return key + ' - <a href="http://pgpkeys.pca.dfn.de:11371/pks/lookup?' +\
               'search=0x%s&fingerprint=on&op=vindex">(details)</a>' % short

    def formatIRCName(self):
        if not self.ircname:
            return ''
        username, server = self.ircname.split('@')
        if server == 'irc.freenode.net':
            server = '<a href="http://freenode.net">%s</a>' % server
        return '%s <strong>auf</strong> %s' % (username, server)
        
    def formatMail(self):
        def format(string):
            string = string.replace('.', ' <strong>dot</strong> ')
            string = string.replace('-', ' <strong>dash</strong> ')
            string = string.replace('_', ' <strong>underline</strong> ')
            return string
        
        if not self.mail: return ''
        username, domain = self.mail.split('@')
        strmail = format(username) + ' <strong>at</strong> ' + format(domain)
        linkid = 'mail-' + md5(str(random())).hexdigest()
        
        return ''.join([
            '<a href="#" id="%s">%s</a>' % (linkid, strmail),
            '<script type="text/javascript">',
            ' el = document.getElementById("%s");' % linkid,
            ' el.href = "mail" + "to:" + "%s" + "@" + "%s";' % (username, domain),
            ' el.innerHTML = "%s" + "@" + "%s";' % (username, domain),
            '</script>'
        ])
    
def create_tables():
    Mitglied.createTable()
    
def test():
    Mitglied(
        uid='armin-ronacher',
        name='Ronacher Armin',
        password=md5('default').hexdigest(),
        GPGkeyid='8E8407E0454AE60096AAA7CB16952A39B5E4A6B4',
        username='blackbird@ubuntuusers.de',
        ircname='mitsuhiko@irc.freenode.net',
        mail='armin.ronacher@active-4.com',
        telefon='+43 676 8423 83 150',
        mitgliedsart=0,
        bezahlt_bis=datetime(2006, 11, 14),
        adresse='Armin Ronacher\nKhünburg, 86\nA-9620 Hermagor',
        kontonummer='kommt hier nicht rein',
        mahnstufe=0,
        private_infos=16,
        is_admin=True
    )
    Mitglied(
        uid='max-mustermann',
        name='Mustermann Max',
        password=md5('default').hexdigest(),
        GPGkeyid='756756756375675637567',
        username='mustermax@ubuntuusers.de',
        ircname='mmustermann@irc.freenode.net',
        mail='max@mustermann.de',
        telefon='n.A.',
        mitgliedsart=0,
        bezahlt_bis=datetime(2006, 1, 1),
        adresse='n.A.',
        kontonummer='n.A.',
        mahnstufe=0,
        private_infos=2,
        is_admin=False
    )
